"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * This file contains Gulp tasks for exporting and importing XLF files.
 * XLF is the interchange format used for requesting and consuming new translations.
 */
const utils_1 = require("@vs-vcremote-loc/utils");
const es = require("event-stream");
const fs = require("fs");
const gulp = require("gulp");
const path = require("path");
const stream_1 = require("stream");
const nls = require("vscode-nls-dev");
const vscode_nls_dev_1 = require("vscode-nls-dev");
const languages_1 = require("./src/languages");
const projectName = "vs-vcremote";
const extensionName = "vs-vcremote";
const vcremoteLocalizationRoot = path.resolve(__dirname, "..", "..");
const packageRoot = path.resolve(__dirname);
const packageSrcRoot = path.resolve(packageRoot, "src");
const outputXlfFile = path.resolve(vcremoteLocalizationRoot, `${projectName}-localization-export`, projectName, `${extensionName}.xlf`);
const inputXlfDir = process.env.LOC_ROOT
    ? process.env.LOC_ROOT
    : path.join(vcremoteLocalizationRoot, "loc");
const i18nDir = path.resolve(packageRoot, "i18n");
const filesToTranslate = [
    "resources.json"
].map((name) => path.resolve(packageSrcRoot, name));
function asModuleName(resourcePath) {
    return path
        .relative(packageSrcRoot, resourcePath)
        .replace(/\\/g, "/")
        .replace(".json", "");
}
function addResourcesToXlf(moduleName, data, xlf) {
    const keys = [];
    const messages = [];
    Object.entries(data).forEach(([key, message]) => {
        if (key.endsWith(".__comment__")) {
            return;
        }
        const commentKey = `${key}.__comment__`;
        keys.push(commentKey in data ? { key, comment: [data[commentKey]] } : key);
        messages.push(message);
    });
    xlf.addFile(moduleName, keys, messages);
}
/**
 * Export all resources defined in this package into a single XLF file.
 */
function exportXlf() {
    const xlf = new vscode_nls_dev_1.XLF(projectName);
    filesToTranslate.forEach((filePath) => {
        addResourcesToXlf(asModuleName(filePath), JSON.parse(fs.readFileSync(filePath, { encoding: "utf8" })), xlf);
    });
    fs.mkdirSync(path.dirname(outputXlfFile), { recursive: true });
    utils_1.Logging.info(`Writing ${outputXlfFile}`);
    fs.writeFileSync(outputXlfFile, xlf.toString(), {
        encoding: "utf8",
    });
}
/**
 * Returns a stream that transforms inbound XLF files into JSON files suitable
 * for use with i18next. This stream is meant for consumption as part of a
 * gulp task.
 */
function importXlfStream() {
    const postprocess = () => new stream_1.Transform({
        objectMode: true,
        transform(file, encoding, callback) {
            if (!file.isBuffer()) {
                throw new Error("Expected file to be buffer");
            }
            // strip leading comments because they are not valid JSON
            // See vscode-nls-dev for hardcoded comment:
            // https://github.com/microsoft/vscode-nls-dev/blob/3c1c16e7f1bae7717d29a6ef2fa2849758c56cec/src/main.ts#L610-L615
            const lines = file.contents.toString().split("\n");
            const numToSkip = (0, utils_1.takeWhile)(lines, (line) => /^(?:\/\*| \*|\/\/)/.test(line)).length;
            lines.splice(0, numToSkip);
            file.contents = Buffer.from(lines.join("\n"));
            // remove i18n from filename
            file.path = file.path.replace("i18n.", "");
            return callback(null, file);
        },
    });
    return es.merge((0, languages_1.translatedLanguages)().map((language) => {
        const xlfPath = path.join(inputXlfDir, language.transifex, "vcremote/vcremote/localization", `${projectName}-localization-export`, projectName, `${extensionName}.${language.transifex}.xlf`);
        const i18nPath = path.join(i18nDir, language.visualStudioDirectory);
        if (fs.existsSync(xlfPath)) {
            utils_1.Logging.info("Reading XLF file: ", xlfPath);
            utils_1.Logging.info("Output Path: ", i18nPath);
        }
        else {
            utils_1.Logging.info("XLF file not found: ", xlfPath);
        }
        return (gulp
            .src(xlfPath, { allowEmpty: true })
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            .pipe(nls.prepareJsonFiles())
            .pipe(postprocess())
            .pipe(gulp.dest(i18nPath)));
    }));
}
gulp.task("translations-export", (done) => {
    exportXlf();
    done();
});
gulp.task("translations-import", (done) => {
    importXlfStream().pipe(es.wait(() => {
        done();
    }));
});
