"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.stringCompare = exports.changeLanguage = exports.localize = exports.initialize = void 0;
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const utils_1 = require("@vs-vcremote-loc/utils");
const i18next_1 = require("i18next");
const languages_1 = require("./languages");
const resources_json_1 = require("./resources.json");
/**
 * Backend for i18next that lazy-loads a resource file only when the language and namespace are requested.
 */
const asyncImportBackend = {
    type: "backend",
    init: () => { },
    read: (language, namespace, callback) => __awaiter(void 0, void 0, void 0, function* () {
        utils_1.Logging.debug(`Loading resources from lang=${language}, ns=${namespace}`);
        try {
            const lang = languages_1.languages.find((l) => l.getCode(languages_1.LanguageCodeKind.VisualStudioUICulture) === language);
            (0, utils_1.assertNotNull)(lang);
            const { default: resources } = yield Promise.resolve(`${`../i18n/${lang.visualStudioDirectory}/${namespace}.json`}`).then(s => require(s));
            callback(null, resources);
        }
        catch (_a) {
            // if language module isn't available, fallback to en-US
            try {
                // Fallback is expected for English.
                if (language !== "en" && language !== "en-US") {
                    utils_1.Logging.warn(`Falling back to en-US when attempting to load lang=${language}, ns=${namespace}`);
                }
                callback(null, resources_json_1.default);
            }
            catch (error) {
                utils_1.Logging.error(error);
                callback(error, null);
            }
        }
    }),
};
let initialized = false;
/**
 * Initialize localization. Called once globally.
 */
function initialize() {
    return __awaiter(this, void 0, void 0, function* () {
        utils_1.Logging.debug("Initializing localization");
        if (initialized) {
            utils_1.Logging.debug("Localization already initialized, skipping.");
            return;
        }
        initialized = true;
        yield i18next_1.default.use(asyncImportBackend).init({
            // make resources the default (and only) namespace
            ns: ["resources"],
            // use flat JSON with no nesting
            keySeparator: false,
            // use {0}, {1} format instead of default {{0}}, {{1}} format
            interpolation: {
                prefix: "{",
                suffix: "}",
            },
            // default language is en-US
            lng: "en-US",
            // fallback to en-US
            fallbackLng: "en-US",
        });
    });
}
exports.initialize = initialize;
/**
 * Return the localized string for the given key. If the string contains interpolation
 * tokens in the form {0}, {1}, ..., they will be replaced with the suppplied replacement strings.
 *
 * @param key Resource key.
 * @param replacements Zero or more strings to be used as token replacements.
 * @param options Optional options to customize localization.
 * @returns The localized, interpolated string for the given key.
 */
function localize(key, replacements, options = {}) {
    var _a;
    if (!initialized) {
        throw new Error("Must initialize localization before requesting strings");
    }
    const dict = {};
    replacements === null || replacements === void 0 ? void 0 : replacements.forEach((s, i) => {
        dict[i.toString()] = s;
    });
    return i18next_1.default.t(key, Object.assign(Object.assign({}, dict), { interpolation: { escapeValue: (_a = options.escapeValue) !== null && _a !== void 0 ? _a : true } }));
}
exports.localize = localize;
/**
 * Change the display language.
 *
 * @param language The language code. Case insensitive.
 * @param kind The language code kind. Defaults to {@link LanguageCodeKind.ISO}.
 */
function changeLanguage(language, kind) {
    return __awaiter(this, void 0, void 0, function* () {
        const found = languages_1.languages.find((lang) => { var _a; return ((_a = lang.getCode(kind)) === null || _a === void 0 ? void 0 : _a.toLowerCase()) === language.toLowerCase(); });
        if (!found) {
            throw new Error(`Unknown language ${language} with kind ${kind}`);
        }
        yield i18next_1.default.changeLanguage(found.visualStudioUICulture);
    });
}
exports.changeLanguage = changeLanguage;
/**
 * Compare strings using the current locale.
 * @param value1 String to call localeCompare on. First string of comparison.
 * @param value2 Parameter to localeCompare call. Second string of comparison.
 * @returns
 */
function stringCompare(value1, value2) {
    return value1.localeCompare(value2, i18next_1.default.language);
}
exports.stringCompare = stringCompare;
