"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultLanguage = exports.translatedLanguages = exports.languages = exports.Language = exports.LanguageCodeKind = void 0;
/**
 * Visual Studio, VS Code, and OneLocBuild use different identifiers for the languages they support.
 *
 * ## VS Code
 *
 * VS Code uses lower-case IETF BCP 47 Language Tags, where only some have the optional region subtag.
 *
 * From https://code.visualstudio.com/docs/getstarted/locales#_available-locales
 *
 * | Display Language    | Locale |
 * | ------------------- | ------ |
 * | English (US)        | en     |
 * | Simplified Chinese  | zh-cn  |
 * | Traditional Chinese | zh-tw  |
 * | French              | fr     |
 * | German              | de     |
 * | Italian             | it     |
 * | Spanish             | es     |
 * | Japanese            | ja     |
 * | Korean              | ko     |
 * | Russian             | ru     |
 * | Bulgarian           | bg     |
 * | Hungarian           | hu     |
 * | Portuguese (Brazil) | pt-br  |
 * | Turkish             | tr     |
 * | Polish              | pl     |
 * | Czech               | cs     |
 *
 * ## Visual Studio
 *
 * Visual Studio's directory structure uses Windows three-letter language names as
 * would be returned by `CultureInfo.ThreeLetterWindowsLanguageName`, documented here
 * https://docs.microsoft.com/en-us/dotnet/api/system.globalization.cultureinfo.threeletterwindowslanguagename?view=net-5.0.
 * Visual Studio does not use consistent casing for these directories, but in this repo we use lower-case.
 * See https://devdiv.visualstudio.com/DevDiv/_git/Localize for an example of these directories.
 *
 * At runtime in Visual Studio, we detect the current culture with `CultureInfo.CurrentUICulture.Name`, which
 * always has the format languagecode2-country/regioncode2. The country/regioncode2 is upper-case.
 *
 * | Display Language    | Win 3-Letter | UICulture.Name |
 * | ------------------- | ------------ | -------------- |
 * | English (US)        | enu          | en-US          |
 * | Simplified Chinese  | chs          | zh-CN          |
 * | Traditional Chinese | cht          | zh-TW          |
 * | French              | fra          | fr-FR          |
 * | German              | deu          | de-DE          |
 * | Italian             | ita          | it-IT          |
 * | Spanish             | esn          | es-ES          |
 * | Japanese            | jpn          | ja-JP          |
 * | Korean              | kor          | ko-KR          |
 * | Russian             | rus          | ru-RU          |
 * | Portuguese (Brazil) | ptb          | pt-BR          |
 * | Turkish             | trk          | tr-TR          |
 * | Polish              | plk          | pl-PL          |
 * | Czech               | csy          | cs-CZ          |
 *
 * ## Transifex
 *
 * The loc pipeline use yet another set of language codes to hand back translations. They mostly
 * coincide with the VS Code language codes, except for the following exceptions.
 *
 * | Display Language    | Locale   |
 * | ------------------- | -------- |
 * | Simplified Chinese  | zh-Hans  |
 * | Traditional Chinese | zh-Hant  |
 * | Portuguese (Brazil) | pt-BR    |
 *
 */
const utils_1 = require("@vs-vcremote-loc/utils");
/** Different language code standards. */
var LanguageCodeKind;
(function (LanguageCodeKind) {
    /** VS Code language identifier */
    LanguageCodeKind["VSCode"] = "VSCode";
    /** Visual Studio language directory names */
    LanguageCodeKind["VisualStudioDirectory"] = "VisualStudioDirectory";
    /** Visual Studio UI culture name */
    LanguageCodeKind["VisualStudioUICulture"] = "VisualStudioUICulture";
    /** Transifex language identifier */
    LanguageCodeKind["Transifex"] = "Transifex";
})(LanguageCodeKind = exports.LanguageCodeKind || (exports.LanguageCodeKind = {}));
/** Language code correspondences for a single language. */
class Language {
    constructor(
    /** VS Code language identifier */
    vscode, 
    /** Visual Studio language directory names */
    visualStudioDirectory, 
    /** Visual Studio UI culture name */
    visualStudioUICulture, 
    /** Transifex language identifier */
    transifex, 
    /** `true` iff this is the default language. Default `false`. */
    isDefault = false) {
        this.vscode = vscode;
        this.visualStudioDirectory = visualStudioDirectory;
        this.visualStudioUICulture = visualStudioUICulture;
        this.isDefault = isDefault;
        this.transifex = transifex !== null && transifex !== void 0 ? transifex : vscode;
    }
    getCode(kind) {
        switch (kind) {
            case LanguageCodeKind.VSCode:
                return this.vscode;
            case LanguageCodeKind.VisualStudioDirectory:
                return this.visualStudioDirectory;
            case LanguageCodeKind.VisualStudioUICulture:
                return this.visualStudioUICulture;
            case LanguageCodeKind.Transifex:
                return this.transifex;
            default:
                throw new Error(`Unrecognized language code kind ${kind}`);
        }
    }
}
exports.Language = Language;
/**
 * List of supported languages.
 *
 * Copied from vscode-cmake-tools sources:
 * @see {@link https://github.com/microsoft/vscode-cmake-tools/blob/e2d75bd1b73649cfdd1ee1317795bc19901c5b96/gulpfile.js#L29}
 *
 */
// prettier-ignore
exports.languages = [
    //           VSCode   VisualStudioDirectory  VisualStudioUICulture  Transifex  default?
    new Language("en", "en", "en-US"),
    new Language("zh-tw", "zh-tw", "zh-TW", "zh-hant"),
    new Language("zh-cn", "zh-cn", "zh-CN", "zh-hans"),
    new Language("fr", "fr", "fr-FR"),
    new Language("de", "de", "de-DE"),
    new Language("it", "it", "it-IT"),
    new Language("es", "es", "es-ES"),
    new Language("ja", "ja", "ja-JP"),
    new Language("ko", "ko", "ko-KR"),
    new Language("ru", "ru", "ru-RU"),
    //new Language("bg",    "bg",                 "bg-BG"), // VS Code supports Bulgarian, but loc team is not currently generating it
    //new Language("hu",    "hu",                 "hu-HU"), // VS Code supports Hungarian, but loc team is not currently generating it
    new Language("pt-br", "pt-br", "pt-BR", "pt-BR"),
    new Language("tr", "tr", "tr-TR"),
    new Language("cs", "cs", "cs-CZ"),
    new Language("pl", "pl", "pl-PL"),
];
/** Returns all languages, even the deault. */
const translatedLanguages = () => exports.languages.filter((lang) => lang.visualStudioDirectory != null);
exports.translatedLanguages = translatedLanguages;
/** The default language */
function defaultLanguage() {
    const found = exports.languages.find((lang) => lang.isDefault);
    (0, utils_1.assertNotNull)(found);
    return found;
}
exports.defaultLanguage = defaultLanguage;
