"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.debug = exports.error = exports.warn = exports.info = exports.addListener = exports.setRedirectionLevel = exports.setLevel = exports.Level = void 0;
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
const events_1 = require("events");
const utils_1 = require("./utils");
var Level;
(function (Level) {
    Level[Level["OFF"] = 0] = "OFF";
    Level[Level["ERROR"] = 1] = "ERROR";
    Level[Level["WARN"] = 2] = "WARN";
    Level[Level["INFO"] = 3] = "INFO";
    Level[Level["DEBUG"] = 4] = "DEBUG";
})(Level = exports.Level || (exports.Level = {}));
let _level = Level.DEBUG;
let _redirectionLevel = undefined;
const loggingEmitter = new events_1.EventEmitter();
const loggingEventName = "logging";
function setLevel(level) {
    _level = level;
}
exports.setLevel = setLevel;
/**
 * If level is defined, messages logged at lower (more severe) levels than level
 * will be redirected as if they were logged at level. This is useful for
 * keeping stderr clean in tests for failure cases. If level is undefined,
 * logging returns to default behavior with no redirection.
 * @param level Level to redirect more severe logging messages to.
 */
function setRedirectionLevel(level) {
    _redirectionLevel = level;
}
exports.setRedirectionLevel = setRedirectionLevel;
function addListener(listenerMethod) {
    loggingEmitter.addListener(loggingEventName, listenerMethod);
}
exports.addListener = addListener;
const info = (...args) => logAtLevel(Level.INFO, args);
exports.info = info;
const warn = (...args) => logAtLevel(Level.WARN, args);
exports.warn = warn;
const error = (...args) => logAtLevel(Level.ERROR, args);
exports.error = error;
const debug = (...args) => logAtLevel(Level.DEBUG, args);
exports.debug = debug;
const logFunctions = new Map([
    [Level.ERROR, console.error],
    [Level.WARN, console.warn],
    [Level.INFO, console.info],
    [Level.DEBUG, console.debug],
]);
function logAtLevel(level, args) {
    warnIfStringReplacement(...args);
    if (_redirectionLevel && level < _redirectionLevel)
        level = _redirectionLevel;
    if (_level >= level) {
        loggingEmitter.emit(loggingEventName, level, ...args.map(utils_1.toStringSafe));
        const func = logFunctions.get(level);
        (0, utils_1.assertNotNull)(func);
        func(...args);
    }
}
function warnIfStringReplacement(...args) {
    args.forEach(function (value) {
        if (value !== undefined && typeof value === "string") {
            if (value.includes("%o") ||
                value.includes("%O") ||
                value.includes("%d") ||
                value.includes("%i") ||
                value.includes("%s") ||
                value.includes("%f") ||
                value.includes("%c")) {
                const errorMessage = "String substitution is not supported";
                console.warn(errorMessage, ...args);
                loggingEmitter.emit(loggingEventName, Level.WARN, errorMessage, ...args);
            }
        }
    });
}
