"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPullRequest = exports.gitExtraArgs = void 0;
const build_settings_1 = require("@vs-vcremote-loc/build-settings");
const utils_1 = require("@vs-vcremote-loc/utils");
const azure_devops_node_api_1 = require("azure-devops-node-api");
const GitInterfaces_1 = require("azure-devops-node-api/interfaces/GitInterfaces");
const child_process_1 = require("child_process");
const crypto_1 = require("crypto");
const path_1 = require("path");
const localizationRoot = path_1.default.resolve(__dirname, "..", "..", "..");
const i18nRoot = path_1.default.resolve(localizationRoot, "app", "localization", "i18n");
// Security token is automatically created for each CI run:
// https://docs.microsoft.com/en-us/azure/devops/pipelines/build/variables?view=azure-devops&tabs=yaml#systemaccesstoken
// For local testing, set the SYSTEM_ACCESSTOKEN environment variable to a PAT with write access
(0, utils_1.assertNotNull)(process.env.SYSTEM_ACCESSTOKEN);
const buildIdentifier = build_settings_1.settings.buildNumber
    ? // For CI, we use the build number
        build_settings_1.settings.buildNumber
    : // For local testing, we use "localtest-<4 bytes of random hex>"
        `localtest-${(0, crypto_1.randomBytes)(4).toString("hex")}`;
const prTitlePrefix = "[Loc Auto-PR]";
const settings = {
    orgUrl: "https://dev.azure.com/devdiv",
    project: "DevDiv",
    repo: "Vcremote",
    sourceBranch: `localization/${buildIdentifier}`,
    targetBranch: "main",
    token: process.env.SYSTEM_ACCESSTOKEN,
    username: "vcremotebot",
    email: "vcremotebot@microsoft.com",
    title: `${prTitlePrefix} ${buildIdentifier}`,
};
// https://docs.microsoft.com/en-us/azure/devops/organizations/accounts/use-personal-access-tokens-to-authenticate?view=azure-devops&tabs=preview-page#use-a-pat
function gitAuthHeader(token, redact) {
    const encodedPat = redact
        ? "**********"
        : Buffer.from(`:${token}`).toString("base64");
    return `http.extraHeader="Authorization: Basic ${encodedPat}"`;
}
function gitExtraArgs(username, email, token, redact) {
    const args = [`-c user.email="${email}"`, `-c user.name="${username}"`];
    if (token) {
        args.push(`-c ${gitAuthHeader(token, redact)}`);
    }
    return args.join(" ");
}
exports.gitExtraArgs = gitExtraArgs;
class Session {
    constructor(settings) {
        this.settings = settings;
        this.initialized = false;
    }
    getWebApi() {
        return __awaiter(this, void 0, void 0, function* () {
            const authHandler = (0, azure_devops_node_api_1.getPersonalAccessTokenHandler)(this.settings.token);
            return new azure_devops_node_api_1.WebApi(settings.orgUrl, authHandler);
        });
    }
    init() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.initialized) {
                console.warn("Already initialized");
                return;
            }
            this.initialized = true;
            this.api = yield this.getWebApi();
            this.git = yield this.api.getGitApi();
        });
    }
    getRepos() {
        return __awaiter(this, void 0, void 0, function* () {
            (0, utils_1.assertNotNull)(this.git);
            return yield this.git.getRepositories(this.settings.project);
        });
    }
    getTargetRepo() {
        return __awaiter(this, void 0, void 0, function* () {
            const repo = (yield this.getRepos()).find((repo) => repo.name === this.settings.repo);
            (0, utils_1.assertNotNull)(repo);
            return repo;
        });
    }
    getRemoteUrl() {
        const { orgUrl, project, repo } = this.settings;
        return `${orgUrl}/${project}/_git/${repo}`;
    }
    runGit(command, withAuth = false) {
        const redactedCliString = `git ${gitExtraArgs(this.settings.username, this.settings.email, withAuth ? this.settings.token : "", true)} ${command}`;
        console.log(redactedCliString);
        const cliString = `git ${gitExtraArgs(this.settings.username, this.settings.email, withAuth ? this.settings.token : "")} ${command}`;
        const result = child_process_1.default.execSync(cliString).toString("utf8");
        console.log(result);
        return result;
    }
    sourceBranchAlreadyExists() {
        try {
            this.runGit(`show-ref --verify --quiet refs/heads/${this.settings.sourceBranch}`);
        }
        catch (error) {
            // --verify returns error code 1 when branch exists
            return error.status !== 1;
        }
        return true;
    }
    // Adapted from
    // https://github.com/microsoft/vscode-cmake-tools/blob/a057e5ceb3ce2f98c01f6eb8117d098101f5f234/translations_auto_pr.js#L56
    hasStagedChanges() {
        console.log("Checking if any files have changed");
        const output = this.runGit("diff --staged --name-only");
        const lines = output.toString().split("\n");
        let anyChanges = false;
        lines.forEach((line) => {
            if (line !== "") {
                console.log("Change detected: " + line);
                anyChanges = true;
            }
        });
        return anyChanges;
    }
    prepareSourceBranch() {
        if (this.sourceBranchAlreadyExists()) {
            throw new Error(`Source branch ${this.settings.sourceBranch} already exists`);
        }
        // remember original branch
        const origBranch = this.runGit(`rev-parse --abbrev-ref HEAD`).trim();
        try {
            // create source branch
            this.runGit(`checkout -b ${this.settings.sourceBranch}`);
            // add all modified i18n files
            this.runGit(`add ${i18nRoot}`);
            // add pnpm-lock.yaml if there are any changes.
            this.runGit(`add ../../../../../common/config/rush/pnpm-lock.yaml`);
            // if no changes were detected, then there are no new translations
            if (!this.hasStagedChanges()) {
                throw new Error("No staged changes detected");
            }
            // commit changes
            this.runGit(`commit -m "${this.settings.title}"`);
            // add remote (remove if it already exists)
            this.runGit(`remote add target ${this.getRemoteUrl()}`);
            // push source branch to remote
            this.runGit(`push target ${this.settings.sourceBranch}`, true);
        }
        finally {
            // restore original commit ID
            this.runGit(`checkout ${origBranch}`);
        }
    }
    activePullRequestExists() {
        return __awaiter(this, void 0, void 0, function* () {
            (0, utils_1.assertNotNull)(this.git);
            const repoId = (yield this.getTargetRepo()).id;
            (0, utils_1.assertNotNull)(repoId);
            const pullRequests = yield this.git.getPullRequests(repoId, {
                status: GitInterfaces_1.PullRequestStatus.Active,
            });
            return pullRequests.some((pr) => { var _a; return (_a = pr.title) === null || _a === void 0 ? void 0 : _a.startsWith(prTitlePrefix); });
        });
    }
    publishPullRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            const request = {
                sourceRefName: `refs/heads/${this.settings.sourceBranch}`,
                targetRefName: `refs/heads/${this.settings.targetBranch}`,
                title: this.settings.title,
                description: "This is an auto-generated PR to update translations.",
            };
            (0, utils_1.assertNotNull)(this.git);
            const repoId = (yield this.getTargetRepo()).id;
            (0, utils_1.assertNotNull)(repoId);
            console.log("Creating Pull Request", request, repoId);
            const response = yield this.git.createPullRequest(request, repoId);
            // Use the createdBy field in the response as the id of the autocompleter
            // https://developercommunity.visualstudio.com/t/how-to-get-identityref-for-the-current-user-or-arb/1129979#T-N1132219
            const createdBy = response.createdBy;
            const pullRequestId = response.pullRequestId;
            (0, utils_1.assertNotNull)(createdBy, "Response is missing expected property: createdBy");
            (0, utils_1.assertNotNull)(pullRequestId, "Response is missing expected property: pullRequestId");
            yield this.git.updatePullRequest({ autoCompleteSetBy: createdBy }, repoId, pullRequestId);
        });
    }
    cleanup() {
        try {
            // remove source branch
            this.runGit(`branch -D ${this.settings.sourceBranch}`);
        }
        catch (error) {
            // ok if the branch doesn't exist
        }
        try {
            // remove remote
            this.runGit(`remote remove target`);
        }
        catch (error) {
            // ok if the remote doesn't exist
        }
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.init();
                if (yield this.activePullRequestExists()) {
                    utils_1.Logging.warn("An active localization pull request already exists, exiting.");
                    return;
                }
                this.prepareSourceBranch();
                yield this.publishPullRequest();
            }
            catch (error) {
                utils_1.Logging.error(error);
            }
            finally {
                this.cleanup();
            }
        });
    }
}
function createPullRequest() {
    void new Session(settings).run();
}
exports.createPullRequest = createPullRequest;
createPullRequest();
